=begin
(c) TIG 2010

  gradienttools.rb goes into Plugins folder. 
  
Usage:

Tag Lines:

Select some stuff including the desired Edges.
Pick Plugins Menu Item 'Gradient Tools...' > 'Lines'
ot type 'gradientline' into the Ruby Console.
Text is added at center of the Edges says something like
'1:60' or '1:25.125' or 'HORIZONTAL' or 'VERTICAL' 
[if no more than 1mm difference between end levels].
Text is put on layer 'GRAD-LINE'.
See below on how to adjust the default text offset 'vec='...
You can move the text-tag using the Text tool - this location remembered on 
Relinking although the arrow will relocate to the Edge's midpoint if appropriate.

Tag Faces:

Select some stuff including the desired Faces.
Pick Plugins Menu Item 'Gradient Tools...' > 'Faces'
ot type 'gradientface' into the Ruby Console.
Text is added at center of the Face says something like
'1:60' or '1:25.125' or 'HORIZONTAL' or 'VERTICAL'
[if no more than 1mm difference between end levels].
Text is put on layer 'GRAD-FACE'.
See below on how to adjust the default text offset 'vec='...
You can move the text-tag using the Text tool - this location remembered on 
Relinking although the arrow will relocate to the Face's center if appropriate.

  Relink Line Tags:
 
This relinks any tagged Edges - using the current gradient for the updated Text
and the current Edge's midpoint for the arrow [relocated text is not moved].
EntityObservers fail on edges/vertices if they are just moved so they can't be used.

  Relink Face Tags:
 
This relinks any tagged Faces - using the current gradient for the updated Text
and the current Face's center for the arrow [relocated text is not moved].

  Relink ALL Tags:
  
This runs 'Relink Line Tags' and 'Relink Face Tags' at the same time.

Donations:
by PayPal.com to info@revitrev.org

Version:
1.0 20100812 First Issue as linegradient.rb
1.1 20100813 Renamed 'gradienttools.rb'.  'Gradient Tools...' submenu added.  
             Split into tools for 'Lines' and 'Faces'.
             Layer GRAD-LINE/GRAD-FACE used. Text-tag now 'tied' to Edge/Face 
             and changes to match the Edge/Face if 'Relink' option used.
1.2 20100813 Downward pointing faces will now report the correct slope.
1.3 20100814 'Relink ALL Tags' menu option added.
             
=end
require 'sketchup.rb'
###
######### LINES #########
def gradientline()
  model=Sketchup.active_model
  ents=model.active_entities
  layers=model.layers
  sel=model.selection
  edges=[]
  sel.each{|e|edges<<e if e.class==Sketchup::Edge}
  if not edges[0]
    UI.messagebox("Gradient Tools... > Tag Lines - ERROR:\n\nSelect some LINES before activating tool!")
    return nil
  end
  model.start_operation("Gradient Tools... > Tag Lines")
  layer=layers.add("GRAD-LINE")
  edges.each{|edge|
    ref=Time::now.to_f.to_i.to_s+rand.to_s
    pst=edge.start.position
    pen=edge.end.position
    pfl=pen.clone; pfl.z=pst.z
    zst=pst.z
    zen=pen.z
    opp=(zst-zen).abs
    adj=pst.distance(pfl)
    if opp< 1.mm
      str="HORIZONTAL"
    elsif adj< 1.mm
      str="VERTICAL"
    else
      zzz=(((adj/opp)*1000.0).to_i).to_f/1000.0 ### to 3 d.p.
      zzz=zzz.to_i if zzz==zzz.to_i
      str="1:"+zzz.to_s
    end#if
    pnt=pst.offset(pst.vector_to(pen),(pst.distance(pen)/2))
    vec=[1, 1, 1] ### change how text offsets from the arrow
    txt=ents.add_text(str,pnt,vec)
    txt.arrow_type=2
    txt.layer=layer
    txt.set_attribute("LineGradient","ref",ref)
    edge.set_attribute("LineGradient","ref",ref)
  }#end each edges
  model.commit_operation
end#def

def gradientlineRelink()
  model=Sketchup.active_model
  model.start_operation("Gradient Tools... > Relink Line Tags")
  edges=[]
  model.entities.each{|e|
    edges<< e if e.class==Sketchup::Edge and e.get_attribute("LineGradient","ref","x")!="x"
  }
  edges.each{|edge|
    ref=edge.get_attribute("LineGradient","ref","x")
    txt=nil
    model.entities.each{|e|
      if e.class==Sketchup::Text and e.get_attribute("LineGradient","ref","y")==ref
        txt=e
        break
      end#if
    }
    next if not txt
    pst=edge.start.position
    pen=edge.end.position
    pfl=pen.clone; pfl.z=pst.z
    zst=pst.z
    zen=pen.z
    opp=(zst-zen).abs
    adj=pst.distance(pfl)
    if opp< 1.mm
      str="HORIZONTAL"
    elsif adj< 1.mm
    else
      zzz=(((adj/opp)*1000.0).to_i).to_f/1000.0 ### to 3 d.p.
      zzz=zzz.to_i if zzz==zzz.to_i
      str="1:"+zzz.to_s
    end#if
    txt.text=str
    pnt=pst.offset(pst.vector_to(pen),(pst.distance(pen)/2))
    txt.point=pnt
    txt.arrow_type=2
  }
  model.commit_operation
end#def

######### FACES #########
def gradientface()
  model=Sketchup.active_model
  ents=model.active_entities
  sel=model.selection
  faces=[]; sel.each{|e|faces.push(e)if e.class==Sketchup::Face}
  if not faces[0]
    UI.messagebox("Gradient Tools... > Tag faces - ERROR:\n\nSelect some FACES before activating tool!")
    return nil
  end#if
  layers=model.layers
  model.start_operation("Gradient Tools... > Tag Faces")
  layer=layers.add("GRAD-FACE")
  faces.each{|face|
    ref=Time::now.to_f.to_i.to_s+rand.to_s
    normal=face.normal
    up=Geom::Vector3d.new(0,0,1)
    angle=normal.angle_between(up)###radians
    angle_d=angle.radians.abs###degrees
    while angle_d>=90
      angle_d=angle_d-90
    end
    if angle_d< 0.01
      str="HORIZONTAL"
    elsif angle_d>= 89.99
      str="VERTICAL"
    else
      itan=1/Math::tan(angle)
      zzz=(((itan)*1000.0).to_i).to_f/1000.0
      str="1:"+zzz.abs.to_s
    end#if
    pnt=face.bounds.center
    vec=[1, 1, 1] ### change how text offsets from the arrow
    txt=ents.add_text(str,pnt,vec)
    txt.arrow_type=2
    txt.layer=layer
    txt.set_attribute("LineGradient","ref",ref)
    face.set_attribute("LineGradient","ref",ref)
  }
  model.commit_operation
  ###  
end#def

def gradientfaceRelink()
  model=Sketchup.active_model
  model.start_operation("Gradient Tools... > Relink Face Tags")
  faces=[]
  model.entities.each{|e|
    faces<< e if e.class==Sketchup::Face and e.get_attribute("LineGradient","ref","x")!="x"
  }
  faces.each{|face|
    ref=face.get_attribute("LineGradient","ref","x")
    txt=nil
    model.entities.each{|e|
      if e.class==Sketchup::Text and e.get_attribute("LineGradient","ref","y")==ref
        txt=e
        break
      end#if
    }
    next if not txt
    normal=face.normal
    up=Geom::Vector3d.new(0,0,1)
    angle=normal.angle_between(up)###radians
    angle_d=angle.radians.abs###degrees
    while angle_d>=90
      angle_d=angle_d-90
    end
    if angle_d< 0.01
      str="HORIZONTAL"
    elsif angle_d>= 89.99
      str="VERTICAL"
    else
      itan=1/Math::tan(angle)
      zzz=(((itan)*1000.0).to_i).to_f/1000.0
      str="1:"+zzz.abs.to_s
    end#if
    txt.text=str
    pnt=face.bounds.center
    txt.point=pnt
    txt.arrow_type=2
  }
  model.commit_operation
end#def

### MENU ###
if not file_loaded?(__FILE__)
  menu=UI.menu("Plugins").add_submenu("Gradient Tools...")
  menu.add_item("Tag Lines"){gradientline()}
  menu.add_item("Tag Faces"){gradientface()}
  menu.add_item("  Relink Line Tags"){gradientlineRelink()}
  menu.add_item("  Relink Face Tags"){gradientfaceRelink()}
  menu.add_item("  Relink ALL Tags"){gradientlineRelink();gradientfaceRelink()}

end
file_loaded(__FILE__)
###